@@ -15,16 +15,30 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-""" Helpers and utilities to apply ANSI coloring to terminal content. """
+"""Helpers and utilities to apply ANSI coloring to terminal content."""
 
+import os
 import re
 import sys
 from collections import namedtuple
+from configparser import RawConfigParser
 from functools import partial
+from operator import getitem
 
 import click
+import regex as re3
+from boltons.strutils import complement_int_list, int_ranges_from_int_list
 from click_log import ColorFormatter
-from cloup import GroupedOption, HelpFormatter, HelpTheme, Style
+
+from . import (
+    Choice,
+    HelpFormatter,
+    HelpTheme,
+    Option,
+    ParameterSource,
+    Style,
+    get_current_context,
+)
 
 # Extend the predefined theme named tuple with our extra styles.
 theme_params = {
@@ -63,13 +77,68 @@ theme = HelpExtraTheme(
 )
 
 
-# Pre-rendered UI-elements.
+# No color theme.
+nocolor_theme = HelpExtraTheme(
+    **{style_id: Style() for style_id in HelpExtraTheme._fields}
+)
+
+
 OK = theme.success("✓")
 KO = theme.error("✘")
+"""Pre-rendered UI-elements."""
+
+color_env_vars = {
+    # Colors.
+    "COLOR": True,
+    "COLORS": True,
+    "CLICOLOR": True,
+    "CLICOLORS": True,
+    "FORCE_COLOR": True,
+    "FORCE_COLORS": True,
+    "CLICOLOR_FORCE": True,
+    "CLICOLORS_FORCE": True,
+    # No colors.
+    "NOCOLOR": False,
+    "NOCOLORS": False,
+    "NO_COLOR": False,
+    "NO_COLORS": False,
+}
+"""List of environment variables recognized as flags to switch color rendering on or off.
+
+The key is the name of the variable and the boolean value the value to pass to ``--color`` option flag when encountered.
+
+Source: https://github.com/pallets/click/issues/558
+"""
 
 
 def disable_colors(ctx, param, value):
-    """Callback disabling all coloring utilities."""
+    """Callback disabling all coloring utilities.
+
+    Re-inspect the environment for existence of colorization flags to re-interpret the
+    provided value.
+    """
+    # Collect all colorize flags in environment variables we recognize.
+    colorize_from_env = set()
+    for var, default in color_env_vars.items():
+        if var in os.environ:
+            # Presence of the variable in the environment without a value encodes for an ativation,
+            # hence the default to True.
+            var_value = os.environ.get(var, "true")
+            # `os.environ` is a dict whose all values are strings. Here we normalize these string into
+            # booleans. If we can't, we fallback to True, in the same spirit as above.
+            var_boolean = RawConfigParser.BOOLEAN_STATES.get(var_value.lower(), True)
+            colorize_from_env.add(default ^ (not var_boolean))
+
+    # Re-interpret the provided value against the recognized environment variables.
+    if colorize_from_env:
+        # The environment can only override the provided value if it comes from
+        # the default value or the config file.
+        env_takes_precedence = (
+            ctx.get_parameter_source("color") == ParameterSource.DEFAULT
+        )
+        if env_takes_precedence:
+            # One env var is enough to activate colorization.
+            value = True in colorize_from_env
 
     # There is an undocumented color flag in context:
     # https://github.com/pallets/click/blob/65eceb08e392e74dcc761be2090e951274ccbe36/src/click/globals.py#L56-L69
@@ -79,7 +148,7 @@ def disable_colors(ctx, param, value):
 
         def restore_original_styling():
             # Reset color flag in context.
-            ctx = click.get_current_context()
+            ctx = get_current_context()
             ctx.color = None
 
         ctx.call_on_close(restore_original_styling)
@@ -93,11 +162,12 @@ def color_option(
     expose_value=False,
     callback=disable_colors,
     help="Strip out all colors and all ANSI codes from output.",
-    cls=GroupedOption,
+    cls=Option,
     **kwargs,
 ):
-    """A ready to use option decorator that is adding a ``--color/--no-color`` (aliased by
-    ``--ansi/--no-ansi``) option to keep or strip colors and ANSI codes from CLI output.
+    """A ready to use option decorator that is adding a ``--color/--no-color`` (aliased
+    by ``--ansi/--no-ansi``) option to keep or strip colors and ANSI codes from CLI
+    output.
 
     This option is eager by defaults to allow for other eager options (like
     ``--version``) to be rendered colorless.
@@ -117,6 +187,13 @@ def color_option(
     )
 
 
+class VersionOption(Option):
+    """No-op class wrapping ``Option`` to serve as a marker to identify parameters
+    created with our own ``version_option`` below."""
+
+    pass
+
+
 def version_option(
     version=None,
     *names,
@@ -127,7 +204,7 @@ def version_option(
     prog_name_style=theme.invoked_command,
     message_style=None,
     env_info_style=Style(fg="bright_black"),
-    cls=GroupedOption,
+    cls=VersionOption,
     **kwargs,
 ):
     """
@@ -171,7 +248,6 @@ def version_option(
     if not colorized_message:
         colorized_message = message
 
-    # XXX:fix to default to user's CLI, not click_extra.__version__).
     return click.version_option(
         version,
         *names,
@@ -206,7 +282,7 @@ class ExtraHelpColorsMixin:
         for param in ctx.command.params:
             options.update(param.opts)
 
-            if isinstance(param.type, click.Choice):
+            if isinstance(param.type, Choice):
                 choices.update(param.type.choices)
 
             if param.metavar:
@@ -241,8 +317,8 @@ class ExtraHelpColorsMixin:
 
 
 class HelpExtraFormatter(HelpFormatter):
-    """Extends Cloup's custom HelpFormatter to highlights options, choices,
-    metavars and default values.
+    """Extends Cloup's custom HelpFormatter to highlights options, choices, metavars and
+    default values.
 
     This is being discussed for upstream integration at:
     * https://github.com/janluke/cloup/issues/97
@@ -266,14 +342,14 @@ class HelpExtraFormatter(HelpFormatter):
     def highlight_extra_keywords(self, help_text):
         """Highlight extra keywoards in help screens based on the theme.
 
-        It is based on regular expressions. While this is not a bullet-proof method, it is good
-        enough. After all, help screens are not consumed by machine but are designed for human.
+        It is based on regular expressions. While this is not a bullet-proof method, it
+        is good enough. After all, help screens are not consumed by machine but are
+        designed for human.
         """
 
         def colorize(match, style):
             """Re-create the matching string by concatenating all groups, but only
-            colorize named groups.
-            """
+            colorize named groups."""
             txt = ""
             for group in match.groups():
                 if group in match.groupdict().values():
@@ -323,6 +399,41 @@ class HelpExtraFormatter(HelpFormatter):
         return help_text
 
     def getvalue(self):
-        """Wrap original `Click.HelpFormatter.getvalue()` to force extra-colorization on rendering."""
+        """Wrap original `Click.HelpFormatter.getvalue()` to force extra-colorization on
+        rendering."""
         help_text = super().getvalue()
         return self.highlight_extra_keywords(help_text)
+
+
+def highlight(string, substrings, styling_method, ignore_case=False):
+    """Highlights parts of the ``string`` that matches ``substrings``.
+
+    Takes care of overlapping parts within the ``string``.
+    """
+    # Ranges of character indices flagged for highlighting.
+    ranges = set()
+
+    for part in set(substrings):
+        # Search for occurrences of query parts in original string.
+        flags = re3.IGNORECASE if ignore_case else 0
+        ranges |= {
+            f"{match.start()}-{match.end() - 1}"
+            for match in re3.finditer(part, string, flags=flags, overlapped=True)
+        }
+
+    # Reduce ranges, compute complement ranges, transform them to list of integers.
+    ranges = ",".join(ranges)
+    highlight_ranges = int_ranges_from_int_list(ranges)
+    untouched_ranges = int_ranges_from_int_list(
+        complement_int_list(ranges, range_end=len(string))
+    )
+
+    # Apply style to range of characters flagged as matching.
+    styled_str = ""
+    for i, j in sorted(highlight_ranges + untouched_ranges):
+        segment = getitem(string, slice(i, j + 1))
+        if (i, j) in highlight_ranges:
+            segment = styling_method(segment)
+        styled_str += segment
+
+    return styled_str
