@@ -15,28 +15,41 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-""" Wrap-up all click-extra extensions into click-like commands.
+"""Wrap-up all click-extra extensions into click-like commands.
 
 The collection of pre-defined decorators here present good and common defaults. You can
 still mix'n'match the mixins below to build your own custom variants.
 """
 
+import io
+from functools import partial
+from logging import getLevelName
 from time import perf_counter
+from unittest.mock import patch
 
 import click
-from cloup import Command, Group, GroupedOption, OptionGroupMixin
-from cloup import command as cloup_command
-from cloup import group as cloup_group
+import cloup
 
-from .colorize import ExtraHelpColorsMixin, color_option, version_option
-from .config import config_option
-from .logging import verbosity_option
+from . import (
+    Command,
+    Group,
+    Option,
+    color_option,
+    config_option,
+    echo,
+    help_option,
+    verbosity_option,
+    version_option,
+)
+from .colorize import ExtraHelpColorsMixin, VersionOption
+from .logging import logger
 
 
 def register_timer_on_close(ctx, param, value):
     """Callback setting up all timer's machinery.
 
-    Computes and print the execution time at the end of the CLI, if option has been activated.
+    Computes and print the execution time at the end of the CLI, if option has been
+    activated.
     """
     # Skip time keeping if option is not active.
     if not value:
@@ -47,7 +60,7 @@ def register_timer_on_close(ctx, param, value):
 
     def print_timer():
         """Compute and print elapsed execution time."""
-        click.echo(f"Execution time: {perf_counter() - timer_start_time:0.3f} seconds.")
+        echo(f"Execution time: {perf_counter() - timer_start_time:0.3f} seconds.")
 
     # Register printing at the end of execution.
     ctx.call_on_close(print_timer)
@@ -59,11 +72,11 @@ def timer_option(
     expose_value=False,
     callback=register_timer_on_close,
     help="Measure and print elapsed execution time.",
-    cls=GroupedOption,
+    cls=Option,
     **kwargs,
 ):
-    """A ready to use option decorator that is adding a ``--time/--no-time``
-    option flag to print elapsed time at the end of CLI execution."""
+    """A ready to use option decorator that is adding a ``--time/--no-time`` option flag
+    to print elapsed time at the end of CLI execution."""
     if not names:
         names = ("--time/--no-time",)
     return click.option(
@@ -112,47 +125,72 @@ class ExtraOptionsMixin:
         version_option(version=version, print_env_info=True)(self)
 
         # Add help option.
-        click.help_option(
-            *self.context_settings["help_option_names"], cls=GroupedOption
-        )(self)
+        help_option(*self.context_settings["help_option_names"], cls=Option)(self)
 
         # Forces re-identification of grouped and non-grouped options.
-        self.option_groups, self.ungrouped_options = self._option_groups_from_params(
+        self.arguments, self.option_groups, self.ungrouped_options = self._group_params(
             self.params
         )
 
     def main(self, *args, **kwargs):
-        """Pre-invokation step that is instanciating the context, then call ``invoke()`` within it.
+        """Pre-invokation step that is instanciating the context, then call ``invoke()``
+        within it.
 
-        During context instanciation, each option's callbacks are called. Beware that these
-        might break the execution flow (like ``--version``).
+        During context instanciation, each option's callbacks are called. Beware that
+        these might break the execution flow (like ``--version``).
         """
         super().main(*args, **kwargs)
 
     def invoke(self, ctx):
-        """Main execution of the command, just after the context has been instanciated in ``main()``."""
+        """Main execution of the command, just after the context has been instanciated
+        in ``main()``.
+
+        Adds, to the normal execution flow, the output of the `--version` parameter in
+        DEBUG logs. This facilitates troubleshooting user's issues.
+        """
+        if getLevelName(logger.level) == "DEBUG":
+
+            # Look for our custom version parameter.
+            for param in ctx.find_root().command.params:
+                if isinstance(param, VersionOption):
+
+                    # Call the --version parameter, but:
+                    #  - capture its output from stdout to redirect it to the DEBUG logger:
+                    #    https://github.com/pallets/click/blob/c1d0729bbb26e3f8b0a28440fb0ebca352535c25/src/click/decorators.py#L451-L455
+                    #    https://github.com/pallets/click/blob/14472ffcd80dd86d47ddc08341168152540ee6f2/src/click/utils.py#L205-L211
+                    capture = io.StringIO()
+                    capture_echo = partial(echo, file=capture)
+                    with patch.object(click.decorators, "echo", new=capture_echo):
+
+                        # Neutralize parameter call to `ctx.exit()`, as seen in:
+                        # https://github.com/pallets/click/blob/c1d0729bbb26e3f8b0a28440fb0ebca352535c25/src/click/decorators.py#L456
+                        with patch(
+                            f"{ctx.__class__.__module__}.{ctx.__class__.__name__}.exit"
+                        ):
+                            param.callback(ctx, param, True)
+
+                    for line in capture.getvalue().splitlines():
+                        logger.debug(line)
+
         return super().invoke(ctx)
 
 
 class ExtraCommand(ExtraHelpColorsMixin, ExtraOptionsMixin, Command):
-    """Cloup default `command` with extra help screen colorization and default options."""
+    """Same as ``cloup.command``, but with extra help screen colorization and
+    options."""
 
     pass
 
 
-class ExtraGroup(ExtraHelpColorsMixin, OptionGroupMixin, ExtraOptionsMixin, Group):
-    """Cloup default `group` with extra help screen colorization and default options.
-
-    Cloup does not support option groups on `Click.group`, hence the inherited `OptionGroupMixin`.
-    This is a workaround that is being discussed at https://github.com/janluke/cloup/issues/98
-    """
+class ExtraGroup(ExtraHelpColorsMixin, ExtraOptionsMixin, Group):
+    """Same as ``cloup.group``, but with extra help screen colorization and options."""
 
     pass
 
 
-def command(name=None, aliases=None, cls=ExtraCommand, **kwargs):
-    return cloup_command(name=name, aliases=aliases, cls=cls, **kwargs)
+def command(*args, cls=ExtraCommand, **kwargs):
+    return cloup.command(*args, cls=cls, **kwargs)
 
 
-def group(name=None, cls=ExtraGroup, **kwargs):
-    return cloup_group(name=name, cls=cls, **kwargs)
+def group(*args, cls=ExtraGroup, **kwargs):
+    return cloup.group(*args, cls=cls, **kwargs)
