@@ -20,17 +20,36 @@ import sys
 from textwrap import dedent
 
 import click
+import cloup
 import pytest
 from boltons.strutils import strip_ansi
-from cloup import Style
-from cloup import command as cloup_command
-from cloup import option, option_group
 
-from .. import __version__
-from ..colorize import HelpExtraFormatter, color_option, theme, version_option
-from ..commands import command, group
-from ..logging import logger, verbosity_option
-from .conftest import skip_windows_colors
+from .. import (
+    Style,
+    __version__,
+    command,
+    echo,
+    group,
+    option,
+    option_group,
+    secho,
+    style,
+    verbosity_option,
+)
+from ..colorize import (
+    HelpExtraFormatter,
+    color_option,
+    highlight,
+    theme,
+    version_option,
+)
+from ..logging import logger
+from .conftest import (
+    default_debug_colored_log,
+    default_debug_uncolored_log,
+    default_options_colored_help,
+    skip_windows_colors,
+)
 
 
 def test_options_highlight():
@@ -95,50 +114,34 @@ def test_keyword_collection(invoke):
         option("-a", "--opt1"),
         option("-b", "--opt2"),
     )
-    @option_group(
+    @cloup.option_group(
         "Group 2",
         option("--opt3"),
         option("--opt4"),
     )
     @option("--test")
     def mycli(opt1, opt2, opt3, opt4, test):
-        click.echo("It works!")
+        echo("It works!")
 
     @command()
     def command1():
-        click.echo("Run click-extra command #1...")
+        echo("Run click-extra command #1...")
 
-    @cloup_command()
+    @cloup.command()
     def command2():
-        click.echo("Run cloup command #2...")
+        echo("Run cloup command #2...")
 
-    @click.command()
+    @click.command
     def command3():
-        click.echo("Run click command #3...")
+        echo("Run click command #3...")
 
     @command()
     def command4():
-        click.echo("Run click-extra command #4...")
+        echo("Run click-extra command #4...")
 
     mycli.section("Subcommand group 1", command1, command2)
     mycli.section("Extra commands", command3, command4)
 
-    default_options_help_screen = (
-        r"  \x1b\[36m--time / --no-time\x1b\[0m        Measure and print elapsed execution time.  \[default:\x1b\[0m\n"
-        r"                            no-time\]\x1b\[0m\n"
-        r"  \x1b\[36m--color, \x1b\[36m--ansi\x1b\[0m / --no-color, --no-ansi\x1b\[0m\n"
-        r"                            Strip out all colors and all ANSI codes from output.\x1b\[0m\n"
-        r"                            \[default: color\]\x1b\[0m\n"
-        r"  \x1b\[36m-C, \x1b\[36m--config\x1b\[0m \x1b\[90mCONFIG_PATH\x1b\[0m\x1b\[0m  Location of the configuration file. Supports both\x1b\[0m\n"
-        r"                            local path and remote URL.  \[default:\x1b\[0m\n"
-        r"(                            \S+\n)*"
-        r"                            \S+config.{toml,yaml,yml,json}\]\x1b\[0m\n"
-        r"  \x1b\[36m-v, \x1b\[36m--verbosity\x1b\[0m \x1b\[90mLEVEL\x1b\[0m\x1b\[0m     Either \x1b\[35mCRITICAL\x1b\[0m, \x1b\[35mERROR\x1b\[0m, \x1b\[35mWARNING\x1b\[0m, \x1b\[35mINFO\x1b\[0m, \x1b\[35mDEBUG\x1b\[0m.\x1b\[0m\n"
-        r"                            \[default: \x1b\[35mINFO\x1b\[0m\]\x1b\[0m\n"
-        r"  \x1b\[36m--version\x1b\[0m                 Show the version and exit.\x1b\[0m\n"
-        r"  \x1b\[36m-h, \x1b\[36m--help\x1b\[0m\x1b\[0m                Show this message and exit.\x1b\[0m\n"
-    )
-
     help_screen = (
         r"\x1b\[94m\x1b\[1m\x1b\[94m\x1b\[1mUsage\x1b\[0m: "
         r"\x1b\[0m\x1b\[97mmycli\x1b\[0m \[OPTIONS\] COMMAND \[ARGS\]...\n\n"
@@ -150,7 +153,7 @@ def test_keyword_collection(invoke):
         r"  \x1b\[36m--opt4 TEXT\x1b\[0m\n\n"
         r"\x1b\[94m\x1b\[1m\x1b\[94m\x1b\[1mOther options\x1b\[0m:\x1b\[0m\n"
         r"  \x1b\[36m--test TEXT\x1b\[0m\n"
-        rf"{default_options_help_screen}"
+        rf"{default_options_colored_help}"
         r"\n"
         r"\x1b\[94m\x1b\[1m\x1b\[94m\x1b\[1mSubcommand group \x1b\[35m1\x1b\[0m\x1b\[0m:\x1b\[0m\n"
         r"  \x1b\[36mcommand1\x1b\[0m\n"
@@ -178,7 +181,7 @@ def test_keyword_collection(invoke):
             r"It works!\n"
             r"\x1b\[94m\x1b\[1m\x1b\[94m\x1b\[1mUsage\x1b\[0m: \x1b\[0m\x1b\[97mmycli command1\x1b\[0m \[OPTIONS\]\n\n"
             r"\x1b\[94m\x1b\[1m\x1b\[94m\x1b\[1mOptions\x1b\[0m:\x1b\[0m\n"
-            rf"{default_options_help_screen}"
+            rf"{default_options_colored_help}"
         ),
         result.output,
     )
@@ -191,7 +194,7 @@ def test_keyword_collection(invoke):
         (
             r"\x1b\[94m\x1b\[1m\x1b\[94m\x1b\[1mUsage\x1b\[0m: \x1b\[0m\x1b\[97mcommand1\x1b\[0m \[OPTIONS\]\n\n"
             r"\x1b\[94m\x1b\[1m\x1b\[94m\x1b\[1mOptions\x1b\[0m:\x1b\[0m\n"
-            rf"{default_options_help_screen}"
+            rf"{default_options_colored_help}"
         ),
         result.output,
     )
@@ -215,10 +218,10 @@ def test_keyword_collection(invoke):
 
 @skip_windows_colors
 def test_standalone_version_option_with_env_info(invoke):
-    @click.group()
+    @click.group
     @version_option(version="1.2.3.4", print_env_info=True)
     def dummy_cli():
-        click.echo("It works!")
+        echo("It works!")
 
     # Test default colouring.
     result = invoke(dummy_cli, "--version", color=True)
@@ -237,10 +240,10 @@ def test_standalone_version_option_with_env_info(invoke):
 
 @skip_windows_colors
 def test_standalone_version_option_without_env_info(invoke):
-    @click.group()
+    @click.group
     @version_option(version="1.2.3.4", print_env_info=False)
     def dummy_cli():
-        click.echo("It works!")
+        echo("It works!")
 
     # Test default colouring.
     result = invoke(dummy_cli, "--version", color=True)
@@ -253,10 +256,10 @@ def test_standalone_version_option_without_env_info(invoke):
 @pytest.mark.parametrize(
     "params", (None, "--help", "blah", ("--config", "random.toml"))
 )
-def test_integrated_version_option_eagerness(invoke, params):
+def test_integrated_version_option_precedence(invoke, params):
     @group(version="1.2.3.4")
     def dummy_cli():
-        click.echo("It works!")
+        echo("It works!")
 
     result = invoke(dummy_cli, "--version", params, color=True)
     assert result.exit_code == 0
@@ -284,19 +287,19 @@ def test_integrated_version_option_eagerness(invoke, params):
     ),
 )
 def test_standalone_color_option(invoke, param, expecting_colors):
-    """Check color option values, defaults and effects on all things colored,
-    including verbosity option."""
+    """Check color option values, defaults and effects on all things colored, including
+    verbosity option."""
 
-    @click.command()
+    @click.command
     @verbosity_option()
     @color_option()
     def dummy_cli():
-        click.echo(Style(fg="yellow")("It works!"))
-        click.echo("\x1b[0m\x1b[1;36mArt\x1b[46;34m\x1b[0m")
-        click.echo(click.style("Run command.", fg="magenta"))
+        echo(Style(fg="yellow")("It works!"))
+        echo("\x1b[0m\x1b[1;36mArt\x1b[46;34m\x1b[0m")
+        echo(style("Run command.", fg="magenta"))
         logger.warning("Processing...")
-        print(click.style("print() bypass Click.", fg="blue"))
-        click.secho("Done.", fg="green")
+        print(style("print() bypass Click.", fg="blue"))
+        secho("Done.", fg="green")
 
     result = invoke(dummy_cli, param, "--verbosity", "DEBUG", color=True)
     assert result.exit_code == 0
@@ -327,19 +330,24 @@ def test_standalone_color_option(invoke, param, expecting_colors):
 
 
 @skip_windows_colors
-def test_color_option_eagerness(invoke):
+def test_color_option_precedence(invoke):
     """--no-color has an effect on --version, if placed in the right order.
 
     Eager parameters are evaluated in the order as they were provided on the command
     line by the user as expleined in:
     https://click.palletsprojects.com/en/8.0.x/advanced/#callback-evaluation-order
+
+    ..todo:
+
+        Maybe have the possibility to tweak CLI callback evaluation order so we can
+        let the user to have the NO_COLOR env set to allow for color-less --version output.
     """
 
-    @click.command()
+    @click.command
     @color_option()
     @version_option(version="2.1.9")
     def dummy_cli():
-        click.echo(Style(fg="yellow")("It works!"))
+        echo(Style(fg="yellow")("It works!"))
 
     result = invoke(dummy_cli, "--no-color", "--version", "command1", color=True)
     assert result.exit_code == 0
@@ -352,6 +360,52 @@ def test_color_option_eagerness(invoke):
     assert not result.stderr
 
 
+@skip_windows_colors
+@pytest.mark.parametrize(
+    "env,env_expect_colors",
+    (
+        ({"COLOR": "true"}, True),
+        ({"COLOR": ""}, True),
+        ({"COLOR": "false"}, False),
+        ({"NO_COLOR": "true"}, False),
+        ({"NO_COLOR": ""}, False),
+        ({"NO_COLOR": "false"}, True),
+        (None, True),
+    ),
+)
+@pytest.mark.parametrize(
+    "param,param_expect_colors",
+    (
+        ("--color", True),
+        ("--no-color", False),
+        ("--ansi", True),
+        ("--no-ansi", False),
+        (None, True),
+    ),
+)
+def test_no_color_env_convention(
+    invoke, env, env_expect_colors, param, param_expect_colors
+):
+    @click.command
+    @color_option()
+    def dummy_cli():
+        echo(Style(fg="yellow")("It works!"))
+
+    result = invoke(dummy_cli, param, color=True, env=env)
+    assert result.exit_code == 0
+    assert not result.stderr
+
+    # Params always overrides env's expectations.
+    expecting_colors = env_expect_colors
+    if param:
+        expecting_colors = param_expect_colors
+
+    if expecting_colors:
+        assert result.output == "\x1b[33mIt works!\x1b[0m\n"
+    else:
+        assert result.output == "It works!\n"
+
+
 @skip_windows_colors
 @pytest.mark.parametrize(
     "param,expecting_colors",
@@ -366,15 +420,15 @@ def test_color_option_eagerness(invoke):
 def test_integrated_color_option(invoke, param, expecting_colors):
     @group()
     def dummy_cli():
-        click.echo(Style(fg="yellow")("It works!"))
-        click.echo("\x1b[0m\x1b[1;36mArt\x1b[46;34m\x1b[0m")
+        echo(Style(fg="yellow")("It works!"))
+        echo("\x1b[0m\x1b[1;36mArt\x1b[46;34m\x1b[0m")
 
     @dummy_cli.command()
     def command1():
-        click.echo(click.style("Run command #1.", fg="magenta"))
+        echo(style("Run command #1.", fg="magenta"))
         logger.warning("Processing...")
-        print(click.style("print() bypass Click.", fg="blue"))
-        click.secho("Done.", fg="green")
+        print(style("print() bypass Click.", fg="blue"))
+        secho("Done.", fg="green")
 
     result = invoke(dummy_cli, param, "--verbosity", "DEBUG", "command1", color=True)
 
@@ -387,12 +441,12 @@ def test_integrated_color_option(invoke, param, expecting_colors):
             "\x1b[34mprint() bypass Click.\x1b[0m\n"
             "\x1b[32mDone.\x1b[0m\n"
         )
-        assert result.stderr == (
-            "\x1b[34mdebug: \x1b[0mVerbosity set to DEBUG.\n"
-            "\x1b[34mdebug: \x1b[0mSearch for configuration in default location...\n"
-            "\x1b[34mdebug: \x1b[0mNo default configuration found.\n"
-            "\x1b[34mdebug: \x1b[0mNo configuration provided.\n"
-            "\x1b[33mwarning: \x1b[0mProcessing...\n"
+        assert re.fullmatch(
+            (
+                rf"{default_debug_colored_log}"
+                r"\x1b\[33mwarning: \x1b\[0mProcessing...\n"
+            ),
+            result.stderr,
         )
 
     else:
@@ -403,10 +457,71 @@ def test_integrated_color_option(invoke, param, expecting_colors):
             "\x1b[34mprint() bypass Click.\x1b[0m\n"
             "Done.\n"
         )
-        assert result.stderr == (
-            "debug: Verbosity set to DEBUG.\n"
-            "debug: Search for configuration in default location...\n"
-            "debug: No default configuration found.\n"
-            "debug: No configuration provided.\n"
-            "warning: Processing...\n"
+        assert re.fullmatch(
+            (rf"{default_debug_uncolored_log}" r"warning: Processing...\n"),
+            result.stderr,
         )
+
+
+@pytest.mark.parametrize(
+    "substrings,expected,ignore_case",
+    (
+        # Function input types.
+        (["hey"], "Hey-xx-xxx-heY-xXxXxxxxx-\x1b[32mhey\x1b[0m", False),
+        (("hey",), "Hey-xx-xxx-heY-xXxXxxxxx-\x1b[32mhey\x1b[0m", False),
+        ({"hey"}, "Hey-xx-xxx-heY-xXxXxxxxx-\x1b[32mhey\x1b[0m", False),
+        (
+            "hey",
+            "H\x1b[32mey\x1b[0m-xx-xxx-\x1b[32mhe\x1b[0mY-xXxXxxxxx-\x1b[32mhey\x1b[0m",
+            False,
+        ),
+        # Duplicate substrings.
+        (["hey", "hey"], "Hey-xx-xxx-heY-xXxXxxxxx-\x1b[32mhey\x1b[0m", False),
+        (("hey", "hey"), "Hey-xx-xxx-heY-xXxXxxxxx-\x1b[32mhey\x1b[0m", False),
+        ({"hey", "hey"}, "Hey-xx-xxx-heY-xXxXxxxxx-\x1b[32mhey\x1b[0m", False),
+        (
+            "heyhey",
+            "H\x1b[32mey\x1b[0m-xx-xxx-\x1b[32mhe\x1b[0mY-xXxXxxxxx-\x1b[32mhey\x1b[0m",
+            False,
+        ),
+        # Case-sensivity and multiple matches.
+        (["hey"], "Hey-xx-xxx-heY-xXxXxxxxx-\x1b[32mhey\x1b[0m", False),
+        (
+            ["Hey"],
+            "\x1b[32mHey\x1b[0m-xx-xxx-\x1b[32mheY\x1b[0m-xXxXxxxxx-\x1b[32mhey\x1b[0m",
+            True,
+        ),
+        (
+            "x",
+            "Hey-\x1b[32mxx\x1b[0m-\x1b[32mxxx\x1b[0m-heY-\x1b[32mx\x1b[0mX\x1b[32mx\x1b[0mX\x1b[32mxxxxx\x1b[0m-hey",
+            False,
+        ),
+        (
+            "x",
+            "Hey-\x1b[32mxx\x1b[0m-\x1b[32mxxx\x1b[0m-heY-\x1b[32mxXxXxxxxx\x1b[0m-hey",
+            True,
+        ),
+        # Overlaps.
+        (
+            ["xx"],
+            "Hey-\x1b[32mxx\x1b[0m-\x1b[32mxxx\x1b[0m-heY-\x1b[32mxXxXxxxxx\x1b[0m-hey",
+            True,
+        ),
+        (
+            ["xx"],
+            "Hey-\x1b[32mxx\x1b[0m-\x1b[32mxxx\x1b[0m-heY-xXxX\x1b[32mxxxxx\x1b[0m-hey",
+            False,
+        ),
+        # No match.
+        ("z", "Hey-xx-xxx-heY-xXxXxxxxx-hey", False),
+        (["XX"], "Hey-xx-xxx-heY-xXxXxxxxx-hey", False),
+    ),
+)
+def test_substring_highlighting(substrings, expected, ignore_case):
+    result = highlight(
+        "Hey-xx-xxx-heY-xXxXxxxxx-hey",
+        substrings,
+        styling_method=theme.success,
+        ignore_case=ignore_case,
+    )
+    assert result == expected
