@@ -15,19 +15,21 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+import re
+
 import click
 import pytest
 
-from ..commands import group
-from ..logging import LOG_LEVELS, logger, verbosity_option
-from .conftest import skip_windows_colors
+from .. import echo, group, verbosity_option
+from ..logging import LOG_LEVELS, logger
+from .conftest import default_debug_colored_log, skip_windows_colors
 
 
 def test_unrecognized_verbosity(invoke):
-    @click.command()
+    @click.command
     @verbosity_option()
     def dummy_cli():
-        click.echo("It works!")
+        echo("It works!")
 
     result = invoke(dummy_cli, "--verbosity", "random")
     assert result.exit_code == 2
@@ -43,10 +45,10 @@ def test_unrecognized_verbosity(invoke):
 @skip_windows_colors
 @pytest.mark.parametrize("level", LOG_LEVELS.keys())
 def test_standalone_verbosity_option(invoke, level):
-    @click.command()
+    @click.command
     @verbosity_option()
     def dummy_cli():
-        click.echo("It works!")
+        echo("It works!")
         logger.debug("my debug message.")
         logger.info("my info message.")
         logger.warning("my warning message.")
@@ -74,21 +76,16 @@ def test_standalone_verbosity_option(invoke, level):
 def test_integrated_verbosity_option(invoke, level):
     @group()
     def dummy_cli():
-        click.echo("It works!")
+        echo("It works!")
 
     @dummy_cli.command()
     def command1():
-        click.echo("Run command #1...")
+        echo("Run command #1...")
 
     result = invoke(dummy_cli, "--verbosity", level, "command1", color=True)
     assert result.exit_code == 0
     assert result.output == "It works!\nRun command #1...\n"
     if level == "DEBUG":
-        assert result.stderr == (
-            "\x1b[34mdebug: \x1b[0mVerbosity set to DEBUG.\n"
-            "\x1b[34mdebug: \x1b[0mSearch for configuration in default location...\n"
-            "\x1b[34mdebug: \x1b[0mNo default configuration found.\n"
-            "\x1b[34mdebug: \x1b[0mNo configuration provided.\n"
-        )
+        assert re.fullmatch(default_debug_colored_log, result.stderr)
     else:
         assert not result.stderr
