@@ -17,14 +17,15 @@
 
 """Test all runner utilities as well as all the test and pytest helpers."""
 
+import os
 from pathlib import Path
 
 import click
-from cloup import Style
 
-from ..commands import command
+from .. import Style, echo, pass_context, secho, style
 from ..logging import logger
 from ..platform import is_windows
+from ..run import env_copy
 from .conftest import skip_windows_colors
 
 
@@ -35,34 +36,45 @@ def test_real_fs():
 
 
 def test_temporary_fs(runner):
-    """Check the CLI runner fixture properly encapsulated the filesystem in
-    temporary directory."""
+    """Check the CLI runner fixture properly encapsulated the filesystem in temporary
+    directory."""
     assert not str(Path(__file__)).startswith(str(Path.cwd()))
 
 
-@click.command()
-@click.pass_context
+def test_env_copy():
+    env_var = "MPM_DUMMY_ENV_VAR_93725"
+    assert env_var not in os.environ
+
+    no_env = env_copy()
+    assert no_env is None
+
+    extended_env = env_copy({env_var: "yo"})
+    assert env_var in extended_env
+    assert extended_env[env_var] == "yo"
+    assert env_var not in os.environ
+
+
+@click.command
+@pass_context
 def dummy_cli(ctx):
-    """https://github.com/pallets/click/issues/2111"""
+    """https://github.com/pallets/click/issues/2111."""
 
-    click.echo(Style(fg="green")("echo()"))
-    click.echo(Style(fg="green")("echo(color=None)"), color=None)
-    click.echo(
-        Style(fg="red")("echo(color=True) bypass invoke.color = False"), color=True
-    )
-    click.echo(Style(fg="green")("echo(color=False)"), color=False)
+    echo(Style(fg="green")("echo()"))
+    echo(Style(fg="green")("echo(color=None)"), color=None)
+    echo(Style(fg="red")("echo(color=True) bypass invoke.color = False"), color=True)
+    echo(Style(fg="green")("echo(color=False)"), color=False)
 
-    click.secho("secho()", fg="green")
-    click.secho("secho(color=None)", fg="green", color=None)
-    click.secho("secho(color=True) bypass invoke.color = False", fg="red", color=True)
-    click.secho("secho(color=False)", fg="green", color=False)
+    secho("secho()", fg="green")
+    secho("secho(color=None)", fg="green", color=None)
+    secho("secho(color=True) bypass invoke.color = False", fg="red", color=True)
+    secho("secho(color=False)", fg="green", color=False)
 
     logger.warning("Is the logger colored?")
 
-    print(click.style("print() bypass Click.", fg="blue"))
+    print(style("print() bypass Click.", fg="blue"))
 
-    click.echo(f"Context.color = {ctx.color!r}")
-    click.echo(f"utils.should_strip_ansi = {click.utils.should_strip_ansi()!r}")
+    echo(f"Context.color = {ctx.color!r}")
+    echo(f"utils.should_strip_ansi = {click.utils.should_strip_ansi()!r}")
 
 
 def check_default_colored_rendering(result):
@@ -143,8 +155,7 @@ def test_invoke_forced_color_stripping(invoke):
 @skip_windows_colors
 def test_invoke_color_keep(invoke):
     """On windows Click ends up deciding it is not running un an interactive terminal
-    and forces the stripping of all colors.
-    """
+    and forces the stripping of all colors."""
     result = invoke(dummy_cli, color=True)
     if is_windows():
         check_default_uncolored_rendering(result)
